# ===============================================================================
# ===============================================================================
#            CropResize Resizer Wrapper Functions 2023-10-28
# ===============================================================================
# ===============================================================================
#
#  This script contains two example resizer wrapper functions for CropResize.
#
# ===============================================================================
#
#  PLEASE READ:
#
#  CropResize needs to import this script in order to use the wrapper functions below.
#  The line for importing CropResizeWrappers.py is at the top of the CropResize script,
#  but it's commented out by default to prevent an error message if CropResizeWrappers.py
#  can't be imported (not everyone will want to use this script).
#
#  Uncommenting the following line at the top of the CropResize script:
#
#  import CropResizeWrappers as crw
#
#  will allow the wrapper functions to be used as CropResize resizers in the manner shown
#  below.
#
#  Resizers in the form of other scripts will have to be imported by CropResize, so a line
#  to import them will need to be added to the top of the CropResize script, much the same
#  way as CropResizeWrappers.py is imported.
#
#  PLEASE NOTE: Any resizers or resizer wrapper functions used with CropResize must accept
#  cropping arguments as float and those arguments must have the same names as the cropping
#  arguments for VapourSynth's resizers. For wrapper functions, the cropping arguments must
#  be included as without the ability to apply cropping with a resizer, the script's cropping
#  and resizing won't be accurate.
#
# ===============================================================================
# ===============================================================================

import vapoursynth as vs
core = vs.core

# ===============================================================================
# ===============================================================================
#            Default Function
# ===============================================================================
# ===============================================================================
#
#  The Default lambda function below serves the same purpose as Avisynth's Default function.
#  It can be used by any of the wrapper functions in this script.
#
# -------------------------------------------------------------------------------

Default = lambda ArgValue, DefaultValue: DefaultValue if (ArgValue is None) else ArgValue

# ===============================================================================
# ===============================================================================
#            Bicubic
# ===============================================================================
# ===============================================================================
#
#   A simple wrapper function for VapourSynth's Bicubic resizer.
#
# -------------------------------------------------------------------------------
#
#   To use it as a resizer for CropResize:
#
#   import CropResize as cr
#   clip = cr.CropResize(clip, 1280,720, Resizer="crw.Bicubic")
#
#   To use it as a stand-alone function:
#
#   import CropResizeWrappers as crw
#   clip = crw.Bicubic(clip, 1280,720)
#
# -------------------------------------------------------------------------------

def Bicubic(clip, width, height, \
    src_left=None, src_top=None, src_width=None, src_height=None, \
    filter_param_a=None, filter_param_b=None):

#   Uncomment the following 2 lines to configure the default values

#   filter_param_a = Default(filter_param_a, 1/3)
#   filter_param_b = Default(filter_param_a, 1/3)

    return core.resize.Bicubic(clip, width, height, \
      src_left=src_left, src_top=src_top, src_width=src_width, src_height=src_height, \
      filter_param_a=filter_param_a, filter_param_b=filter_param_b)

# ===============================================================================
# ===============================================================================
#            FMTConvLinear
# ===============================================================================
# ===============================================================================
#
#   Resizing in linear light with FMTConv
#   http://avisynth.nl/index.php/Fmtconv
#
# -------------------------------------------------------------------------------
#
#   To use it as a resizer for CropResize:
#
#   import CropResize as cr
#   clip = cr.CropResize(clip, 1280,720, Resizer="crw.FMTConvLinear", RStr='kernel="Bicubic"')
#
#   To use it as a stand-alone function:
#
#   import CropResizeWrappers as crw
#   clip = crw.FMTConvLinear(clip, 1280,720, kernel="Bicubic")
#
# -------------------------------------------------------------------------------

def FMTConvLinear(Source, width, height, \
    src_left=None, src_top=None, src_width=None, src_height=None, kernel=None, \
    out_bits=None, mat=None, trans=None, full=None, taps=None, a1=None, a2=None):

    Source_Bits = Source.format.bits_per_sample

# -------------------------------------------------------------------------------
#   Some Defaults
# -------------------------------------------------------------------------------

#   mat  -  the matrix used for converting YUV to linear RGB and back.
#   trans  -  the transfer function for converting YUV to linear RGB and back.
#   full  -  YUV only, true for full range, false for limited range.

    out_bits = Default(out_bits, Source_Bits)
    mat = Default(mat, "709")
    trans = Default(trans, "709")
    full = Default(full, False)
#   kernel = Default(kernel, "spline36")
#   taps = Default(taps, xxx)
#   a1 = Default(a1, xxx)
#   a2 = Default(a2, xxx)

# -------------------------------------------------------------------------------

    def getCSS(w, h):

        css={
            (1, 1): "420",
            (1, 0): "422",
            (0, 0): "444",
            (2, 0): "411"}
        sub=(w, h)

        if css.get(sub) is None:
            raise vs.Error('FMTConvLinear: Usupported chroma subsampling')
        else: return css[sub]

    ChromaSS = getCSS(Source.format.subsampling_w, Source.format.subsampling_h)
    SCFamily = Source.format.color_family

# -------------------------------------------------------------------------------

    Input = Source if (Source_Bits == 16) else Source.fmtc.bitdepth(bits=16)
    Input = Input if (ChromaSS == "444") else core.fmtc.resample(Input, css="444")

    if (SCFamily != vs.RGB):

        Resized = Input.fmtc.matrix(mat=mat, fulls=full, fulld=True, bits=16)
        Resized = Resized.fmtc.transfer(transs=trans, transd="linear")
        Resized = Resized.fmtc.resample(width, height, sx=src_left, sy=src_top, \
          sw=src_width, sh=src_height, kernel=kernel, taps=taps, a1=a1, a2=a2)
        Resized = Resized.fmtc.transfer(transs="linear", transd=trans)
        Resized = Resized.fmtc.matrix(mat=mat, fulls=True, fulld=full, bits=16)

    else:

        Resized = Input.fmtc.transfer(transs="srgb", transd="linear")
        Resized = Resized.fmtc.resample(width, height, sx=src_left, sy=src_top, \
          sw=src_width, sh=src_height, kernel=kernel, taps=taps, a1=a1, a2=a2)
        Resized = Resized.fmtc.transfer(transs="linear", transd="srgb")

    Output = \
      Resized.fmtc.resample(css="422") if (ChromaSS == "422") else \
      Resized.fmtc.resample(css="420") if (ChromaSS == "420") else \
      Resized.fmtc.resample(css="411") if (ChromaSS == "411") else Resized

    return Output if (Output.format.bits_per_sample == out_bits) else \
      Output.fmtc.bitdepth(bits=out_bits)

# ===============================================================================
# ===============================================================================
